Attribute VB_Name = "modMap"
' modMap.bas


Type LoadMapType
    Name As String
    
    Lower As String
    Upper As String
    
    Data(100) As String
End Type


Type MapType
    Name As String
    
    Lower As String
    Upper As String
    
    StartXpos As Long
    StartYpos As Long

End Type

Global Map As MapType

Global ItemMap() As Long
Global PlayerMap() As Long
Global VisibleMap() As Boolean
Global LightMap() As Integer


Function Map_Check(Xpos As Long, Ypos As Long) As Boolean
    ' make sure in bounds
    If Xpos < 0 Or Xpos > UBound(ItemMap) Or Ypos < 0 Or Ypos > UBound(ItemMap) Then Exit Function

    Map_Check = True
    
End Function


Sub modMap_Load(Filename As String)
    On Error GoTo Error_modMap_Load

    Dim LoadMap As LoadMapType
   
    Dim filenum As Integer, s1 As String, dIndex As Long
    
    filenum = FreeFile()
    
    Open Filename For Input As filenum
    
    ' read the file in
    
    Do Until EOF(filenum)
        Line Input #filenum, s1
    
        If Left$(s1, 1) = ";" Or Len(Trim$(s1)) = 0 Then
            ' remark, just ignore
        
        ElseIf UCase$(Left$(s1, 4)) = "MAP=" Then
            LoadMap.Name = Trim$(Right$(s1, Len(s1) - 4))
        
        ElseIf UCase$(Left$(s1, 6)) = "LOWER=" Then
            LoadMap.Lower = Trim$(Right$(s1, Len(s1) - 6))
        
        ElseIf UCase$(Left$(s1, 6)) = "UPPER=" Then
            LoadMap.Upper = Trim$(Right$(s1, Len(s1) - 6))
        
        ElseIf UCase$(Left$(s1, 1)) = "X" Then
            ' map data
            If dIndex > UBound(LoadMap.Data) Or Len(Trim$(s1)) > UBound(LoadMap.Data) Then
                MsgBox "Map too big!", vbCritical, "FATAL ERROR! DIE! DIE!"
            Else
                LoadMap.Data(dIndex) = Trim$(s1)
                dIndex = dIndex + 1
            End If
                
        Else
            ' unknown line found
            MsgBox "Unknown line found in " & Filename & vbCrLf & s1, vbExclamation, "WARNING!!!!"
        
        End If
    
    Loop
    
    Close filenum
    
    modMap_Setup LoadMap
    
    Exit Sub
    
Error_modMap_Load:
    MsgBox "Error_modMap_Load : " & Err.Description
End Sub


Sub modMap_Setup_Init(MapName As String)
    ReDim ItemMap(100, 100)
    ReDim PlayerMap(100, 100)
    ReDim VisibleMap(100, 100)
    ReDim LightMap(100, 100)

    modItem_Init
    
    Dim i As Long, j As Long

    ' reset the players
    For i = 1 To UBound(Player)
        If Player(i).flgInUse Then
            If Game.pIndex = i Then
                PlayerMap(Player(i).Xpos, Player(i).Ypos) = i
            Else
                modPlayer_Remove i
            End If
        End If
    Next i
   
    frmMain.Caption = "Dungeon Fate (ver " & App.Major & "." & App.Minor & "." & App.Revision & ")  : " & MapName

End Sub

Sub modMap_Setup(LoadMap As LoadMapType)
    On Error GoTo Error_modMap_Setup

    modMap_Setup_Init LoadMap.Name
   
    Dim EmptyMap As MapType
    Map = EmptyMap
    
    Map.Name = LoadMap.Name
    Map.Lower = LoadMap.Lower
    Map.Upper = LoadMap.Upper
   
    Dim i As Long, j As Long
   
    For i = 0 To 100
        If LoadMap.Data(i) = "" Then Exit For
        
        For j = 0 To Len(LoadMap.Data(i)) - 1
            Select Case Mid$(LoadMap.Data(i), j + 1, 1)
                Case "X", "x"
                    ' wall
                    modItem_Add 1, j, i
                    
                Case "D", "d"
                    ' door
                    modItem_Add 17, j, i
                    
                Case "g"
                    'small gold pile
                    modItem_Add 24, j, i
                    
                Case "G"
                    ' large gold pile
                    modItem_Add 25, j, i
                    
                Case "e"
                    'stairs down
                    modItem_Add 21, j, i
                                        
                Case "E"
                    'stairs up
                    modItem_Add 22, j, i
                                        
                Case "S", "s"
                    ' start spot
                    Map.StartXpos = j
                    Map.StartYpos = i
                    
                Case "!"
                    ' dagger
                    modItem_Add modItem_ID("dagger"), j, i
                    
                Case "@"
                    ' short sword
                    modItem_Add modItem_ID("short sword"), j, i
                    
                Case "#"
                    ' long sword
                    modItem_Add modItem_ID("long sword"), j, i
                    
                Case "$"
                    ' broad sword
                    modItem_Add modItem_ID("broad sword"), j, i
                    
                Case "%"
                    ' great sword
                    modItem_Add modItem_ID("great sword"), j, i
                    
                Case "["
                    ' soft leather
                    modItem_Add modItem_ID("soft leather armor"), j, i
                    
                Case "]"
                    ' hard leather
                    modItem_Add modItem_ID("hard leather armor"), j, i
                    
                Case "\"
                    ' studded leather
                    modItem_Add modItem_ID("studded leather armor"), j, i
                    
                Case "1" To "9"
                    ' monster
                    modPlayer_Add Int(Mid$(LoadMap.Data(i), j + 1, 1)), j, i
                    
                Case Else
                    ' unknown data
                    
            End Select
        Next j
    Next i

    modMap_Fix_Walls

    Exit Sub
    
Error_modMap_Setup:
    MsgBox "Error_modMap_Setup : " & Err.Description
End Sub


Function modMap_Is_Stone_Wall(Xpos As Long, Ypos As Long) As Boolean
    On Error GoTo Error_Is_Stone_Wall

    modMap_Is_Stone_Wall = False
    
    If Not Map_Check(Xpos, Ypos) Then Exit Function
    
    Dim iIndex As Long
    iIndex = ItemMap(Xpos, Ypos)
    
    If iIndex = 0 Then Exit Function
    
    Select Case Item(iIndex).ID
        Case 1 To 20
            modMap_Is_Stone_Wall = True
    End Select

    Exit Function

Error_Is_Stone_Wall:
    MsgBox "Error_Is_Stone_Wall : " & Err.Description
End Function


Function modMap_Get_Wall_Connector_ID(iIndex As Long)
    On Error GoTo Error_Get_Wall_Connector_ID

    modMap_Get_Wall_Connector_ID = Item(iIndex).ID

    Dim Xpos As Long, Ypos As Long
    Xpos = Item(iIndex).Xpos
    Ypos = Item(iIndex).Ypos

    Dim side2 As Boolean, side4 As Boolean, side6 As Boolean, side8 As Boolean
    
    side6 = modMap_Is_Stone_Wall(Xpos + 1, Ypos)    ' 6
    side4 = modMap_Is_Stone_Wall(Xpos - 1, Ypos)    ' 4
    side2 = modMap_Is_Stone_Wall(Xpos, Ypos + 1)    ' 2
    side8 = modMap_Is_Stone_Wall(Xpos, Ypos - 1)    ' 8
    
    If Item(iIndex).ID >= 17 And Item(iIndex).ID <= 20 Then
        ' door
        
        If side8 And side2 And Not side4 And Not side6 Then
            modMap_Get_Wall_Connector_ID = 19
        ElseIf Not side8 And Not side2 And side4 And side6 Then
            modMap_Get_Wall_Connector_ID = 17
        End If
    
    Else
        ' regular wall
        
        If side2 And side4 And side6 And side8 Then
            modMap_Get_Wall_Connector_ID = 8     ' wall x
        
        ElseIf side2 And side4 And Not side6 And side8 Then
            modMap_Get_Wall_Connector_ID = 11     ' e wall T
        
        ElseIf side2 And Not side4 And side6 And side8 Then
            modMap_Get_Wall_Connector_ID = 10     ' w wall T
        
        ElseIf Not side2 And side4 And side6 And side8 Then
            modMap_Get_Wall_Connector_ID = 3     ' S wall T
    
        ElseIf side2 And side4 And side6 And Not side8 Then
            modMap_Get_Wall_Connector_ID = 8     ' N wall T
    
        ElseIf Not side2 And Not side4 And side6 And side8 Then
            modMap_Get_Wall_Connector_ID = 6
    
        ElseIf Not side2 And side4 And Not side6 And side8 Then
            modMap_Get_Wall_Connector_ID = 7
    
        ElseIf Not side2 And side4 And side6 And Not side8 Then
            modMap_Get_Wall_Connector_ID = 3
    
        ElseIf side2 And Not side4 And Not side6 And side8 Then
            modMap_Get_Wall_Connector_ID = 5
    
        ElseIf side2 And Not side4 And side6 And Not side8 Then
            modMap_Get_Wall_Connector_ID = 2
    
        ElseIf side2 And side4 And Not side6 And Not side8 Then
            modMap_Get_Wall_Connector_ID = 4
    
        ElseIf Not side2 And Not side4 And Not side6 And side8 Then
            modMap_Get_Wall_Connector_ID = 14
    
        ElseIf Not side2 And Not side4 And side6 And Not side8 Then
            modMap_Get_Wall_Connector_ID = 15
    
        ElseIf side2 And Not side4 And Not side6 And Not side8 Then
            modMap_Get_Wall_Connector_ID = 13
    
        ElseIf Not side2 And side4 And Not side6 And Not side8 Then
            modMap_Get_Wall_Connector_ID = 16
            
        End If
    End If
    
    Exit Function
    
Error_Get_Wall_Connector_ID:
    MsgBox "Error_modMap_get_wall_connector : " & Err.Description
End Function


Sub modMap_Fix_Walls()
    On Error GoTo Error_modMap_Fix_Walls
    
    ' looking for granite wall item = 10
    ' then connect with proper walls
    
    Dim i As Long, j As Long
    Dim iIndex As Long
    
    For i = 0 To UBound(ItemMap)
        For j = 0 To UBound(ItemMap)
    
            iIndex = ItemMap(i, j)
            
            If iIndex > 0 Then
                If modMap_Is_Stone_Wall(i, j) Then
                
                    Item(iIndex).ID = modMap_Get_Wall_Connector_ID(iIndex)
                
                End If
            End If
    
        Next j
    Next i

    Exit Sub

Error_modMap_Fix_Walls:
    MsgBox "Error_modMap_Fix_Walls : " & Err.Description
End Sub


Sub modMap_Set_Light(Xext As Long, Yext As Long)
    On Error GoTo Error_modMap_Set_Light

    If Game.pIndex = 0 Then Exit Sub

    Dim i As Long, j As Long
        
    For i = 0 To UBound(LightMap)
        For j = 0 To UBound(LightMap)
            LightMap(i, j) = 7
        Next j
    Next i

    modMap_Add_Light Player(Game.pIndex).Xpos, Player(Game.pIndex).Ypos, 7

    Exit Sub
    
Error_modMap_Set_Light:
    MsgBox "Error_modMap_Set_Light : " & Err.Description
End Sub


Sub modMap_Add_Light(Xpos As Long, Ypos As Long, Light As Integer)
    On Error GoTo Error_modMap_Add_Light

    Dim i As Long, j As Long, Xpos2 As Long, Ypos2 As Long
    
    For i = -Light To Light
        For j = -Light To Light
        
            Xpos2 = Xpos + i
            Ypos2 = Ypos + j
            
            If Map_Check(Xpos2, Ypos2) Then
            
                LightMap(Xpos2, Ypos2) = LightMap(Xpos2, Ypos2) - (Light - modMap_Dist(Xpos, Ypos, Xpos2, Ypos2))
            
                If LightMap(Xpos2, Ypos2) < 0 Then
                    LightMap(Xpos2, Ypos2) = 0
                End If
            End If
        
        Next j
    Next i

    Exit Sub
    
Error_modMap_Add_Light:
    MsgBox "Error_modMap_Add_Light : " & Err.Description
End Sub


Sub modMap_Set_Visible(Xext As Long, Yext As Long)
    On Error GoTo Error_modMap_Set_Visible

    If Game.pIndex = 0 Then Exit Sub

    Dim i As Long, j As Long
    
    For i = 0 To UBound(VisibleMap)
        For j = 0 To UBound(VisibleMap)
            VisibleMap(i, j) = False
        Next j
    Next i

    Dim Xpos As Long, Ypos As Long, k As Long, l As Long
    Dim Xpos2 As Long, Ypos2 As Long

    For i = -Xext To Xext
        For j = -Yext To Yext
            Xpos = Player(Game.pIndex).Xpos + i
            Ypos = Player(Game.pIndex).Ypos + j
            
            If Map_Check(Xpos, Ypos) Then
                
                If modMap_Open_Sight_Line(Player(Game.pIndex).Xpos, Player(Game.pIndex).Ypos, Xpos, Ypos) Then
                    VisibleMap(Xpos, Ypos) = True
                    
                    If modMap_Open_Sight_Line_Helper(Xpos, Ypos) Then
                        For k = -1 To 1
                            For l = -1 To 1
                                Xpos2 = Xpos + k
                                Ypos2 = Ypos + l
                                
                                If Map_Check(Xpos2, Ypos2) Then
                                    VisibleMap(Xpos2, Ypos2) = True
                                End If
                            Next l
                        Next k
                    End If
                    
                End If
            End If
            
        Next j
    Next i

    Exit Sub
    
Error_modMap_Set_Visible:
    MsgBox "Error_modMap_Set_Visible : " & Err.Description
End Sub


Function modMap_Is_Visible(Xpos As Long, Ypos As Long) As Boolean
    On Error GoTo Error_modMap_Is_Visible

    If Not Map_Check(Xpos, Ypos) Then Exit Function
    
    If Not VisibleMap(Xpos, Ypos) Then Exit Function
    
    modMap_Is_Visible = True
    
    Exit Function
    
Error_modMap_Is_Visible:
    MsgBox "Error_modMap_Is_Visible : " & Err.Description
End Function


Function modMap_Open_Sight_Line_Helper(Xpos As Long, Ypos As Long) As Boolean

    Dim iIndex As Long

    If Not Map_Check(Xpos, Ypos) Then Exit Function
        
    iIndex = ItemMap(Xpos, Ypos)

    If iIndex > 0 Then
        If ItemDef(Item(iIndex).ID).flgBlockMovement Then
            Exit Function
        End If
    End If
    
    modMap_Open_Sight_Line_Helper = True
    
End Function

Function modMap_Open_Sight_Line(X1 As Long, Y1 As Long, X2 As Long, Y2 As Long) As Boolean
    On Error GoTo Error_modMap_Open_Sight_Line

    modMap_Open_Sight_Line = True
    
    If modMap_Dist(X1, Y1, X2, Y2) <= 1 Then Exit Function
        
    ' 4 case: x move, y move, vertical, horizontal
    
    Dim XXrun As Integer, YYrise As Integer
    Dim Xstart As Integer, Ystart As Integer
    Dim Xstop As Integer, Ystop As Integer
    Dim StepNum As Integer, pIndex As Integer
    
    XXrun = X1 - X2
    YYrise = Y1 - Y2
    
    If X1 > X2 Then
        Xstart = X2
        Xstop = X1
        'Xstart = X1
        'Xstop = X2
        'StepNum = 1
    Else
        Xstart = X1
        Xstop = X2
        'StepNum = 1
    End If
    
    If Y1 > Y2 Then
        Ystart = Y2
        Ystop = Y1
        'Ystart = Y1
        'Ystop = Y2
        'StepNum = 1
    Else
        Ystart = Y1
        Ystop = Y2
        'StepNum = 1
    End If
    
    Dim Xpos As Long, Ypos As Long, i As Long
    Dim Slope As Single
    Dim iIndex As Long
    
    If XXrun = 0 Then
        ' vertical line
        Xpos = X1
        BlockCount = 0
    
        For i = Ystart To Ystop
            
            If (Xpos <> X1 Or i <> Y1) And (Xpos <> X2 Or i <> Y2) Then
            
                If Not modMap_Open_Sight_Line_Helper(Xpos, i) Then
                    modMap_Open_Sight_Line = False
                    Exit Function
                End If
            
            End If
            
        Next i
        
        Exit Function
    
    End If
    
    If YYrise = 0 Then
        ' horixontal line
    
        Ypos = Y1
        BlockCount = 0
    
        For i = Xstart To Xstop
            
            If (i <> X1 Or Ypos <> Y1) And (i <> X2 Or Ypos <> Y2) Then

                If Not modMap_Open_Sight_Line_Helper(i, Ypos) Then
                    modMap_Open_Sight_Line = False
                    Exit Function
                End If
                
            End If
        
        Next i
        
        Exit Function
    
    End If
    
    If Abs(XXrun) > Abs(YYrise) Then
        ' x move
        
        Slope = YYrise / XXrun
        BlockCount = 0
        
        For i = 0 To Xstop - Xstart
            'Ypos = (CSng(i - Xstart) / CSng(XXrun)) * YYrise + Ystart
            Ypos = Slope * i
            If Slope > 0 Then
                Ypos = Ypos + Ystart
            Else
                Ypos = Ypos + Ystart + Abs(YYrise)
            End If
                
            If (i + Xstart <> X1 Or Ypos <> Y1) And (i + Xstart <> X2 Or Ypos <> Y2) Then
                
                If Not modMap_Open_Sight_Line_Helper(i + Xstart, Ypos) Then
                    modMap_Open_Sight_Line = False
                    Exit Function
                End If
                
            End If
        Next i
        
        Exit Function
    
    End If
    
    If Abs(YYrise) >= Abs(XXrun) Then
        ' y move
    
        Slope = YYrise / XXrun
        BlockCount = 0
        
        For i = 0 To Ystop - Ystart
            Xpos = i / Slope
            
            If Slope > 0 Then
                Xpos = Xpos + Xstart
            Else
                Xpos = Xpos + Xstart + Abs(XXrun)
            End If
                            
            If (Xpos <> X1 Or i + Ystart <> Y1) And (Xpos <> X2 Or i + Ystart <> Y2) Then
                            
                If Not modMap_Open_Sight_Line_Helper(Xpos, i + Ystart) Then
                    modMap_Open_Sight_Line = False
                    Exit Function
                End If
                            
            End If
        Next i
        
        Exit Function
    
    End If
        
    Exit Function

Error_modMap_Open_Sight_Line:
    MsgBox "Error_modMap_Open_Sight_Line : " & Err.Description
End Function



Function modMap_Dist(X1 As Long, Y1 As Long, X2 As Long, Y2 As Long) As Integer
    On Error GoTo Error_modMap_Dist
    
    Dim x As Long, y As Long
    
    x = Abs(X1 - X2)
    y = Abs(Y1 - Y2)
    
    If x > y Then
        modMap_Dist = x
    Else
        modMap_Dist = y
    End If
    
    Exit Function
    
Error_modMap_Dist:
    MsgBox "Error_modMap_Dist : " & Err.Description
End Function


Function modMap_Direction(X1 As Long, Y1 As Long, X2 As Long, Y2 As Long) As Integer
    On Error GoTo Error_modMap_Direction
    
    ' uses the numeric pad again, right

    If X1 > X2 Then
        If Y1 > Y2 Then
            modMap_Direction = 7
        ElseIf Y1 < Y2 Then
            modMap_Direction = 1
        Else
            modMap_Direction = 4
        End If
        
    ElseIf X1 < X2 Then
        If Y1 > Y2 Then
            modMap_Direction = 9
        ElseIf Y1 < Y2 Then
            modMap_Direction = 3
        Else
            modMap_Direction = 6
        End If
    
    Else
        If Y1 > Y2 Then
            modMap_Direction = 8
        ElseIf Y1 < Y2 Then
            modMap_Direction = 2
        Else
            modMap_Direction = 5 ' equal? should never get here
        End If
    End If

    Exit Function
    
Error_modMap_Direction:
    MsgBox "Error_modMap_Direction : " & Err.Description
End Function


Function modMap_AltDirection(Direction As Integer) As Integer
    On Error GoTo Error_modMap_AltDirection

    ' gets a diagonal similiar to original direction

    Select Case Direction
        Case 1
            If Rnd > 0.5 Then
                modMap_AltDirection = 2
            Else
                modMap_AltDirection = 4
            End If
        Case 2
            If Rnd > 0.5 Then
                modMap_AltDirection = 1
            Else
                modMap_AltDirection = 3
            End If
        Case 3
            If Rnd > 0.5 Then
                modMap_AltDirection = 2
            Else
                modMap_AltDirection = 6
            End If
        Case 4
            If Rnd > 0.5 Then
                modMap_AltDirection = 1
            Else
                modMap_AltDirection = 7
            End If
        Case 6
            If Rnd > 0.5 Then
                modMap_AltDirection = 3
            Else
                modMap_AltDirection = 9
            End If
        Case 7
            If Rnd > 0.5 Then
                modMap_AltDirection = 4
            Else
                modMap_AltDirection = 8
            End If
        Case 8
            If Rnd > 0.5 Then
                modMap_AltDirection = 7
            Else
                modMap_AltDirection = 9
            End If
        Case 9
            If Rnd > 0.5 Then
                modMap_AltDirection = 6
            Else
                modMap_AltDirection = 8
            End If
    
    End Select

    Exit Function

Error_modMap_AltDirection:
    MsgBox "Error_modMap_AltDirection : " & Err.Description
End Function


Function modMap_Opposite_Direction(Direction As Integer) As Integer
    On Error GoTo Error_modMap_Opposite_Direction

    Select Case Direction
        Case 1
            modMap_Opposite_Direction = 9
        Case 2
            modMap_Opposite_Direction = 8
        Case 3
            modMap_Opposite_Direction = 7
        Case 4
            modMap_Opposite_Direction = 6
        Case 6
            modMap_Opposite_Direction = 4
        Case 7
            modMap_Opposite_Direction = 3
        Case 8
            modMap_Opposite_Direction = 2
        Case 9
            modMap_Opposite_Direction = 1
    End Select

    Exit Function
    
Error_modMap_Opposite_Direction:
    MsgBox "Error_modMap_Opposite_Direction : " & Err.Description
End Function


Function modMap_Quarter_Direction(Direction As Integer, Turn As Integer) As Integer
    On Error GoTo Error_modMap_Quarter_Direction

    If Turn = 1 Then
        Select Case Direction
            Case 1
                modMap_Quarter_Direction = 7
            Case 2
                modMap_Quarter_Direction = 4
            Case 3
                modMap_Quarter_Direction = 1
            Case 4
                modMap_Quarter_Direction = 8
            Case 6
                modMap_Quarter_Direction = 2
            Case 7
                modMap_Quarter_Direction = 9
            Case 8
                modMap_Quarter_Direction = 6
            Case 9
                modMap_Quarter_Direction = 3
        End Select
    Else
        Select Case Direction
            Case 1
                modMap_Quarter_Direction = 3
            Case 2
                modMap_Quarter_Direction = 6
            Case 3
                modMap_Quarter_Direction = 9
            Case 4
                modMap_Quarter_Direction = 2
            Case 6
                modMap_Quarter_Direction = 8
            Case 7
                modMap_Quarter_Direction = 1
            Case 8
                modMap_Quarter_Direction = 4
            Case 9
                modMap_Quarter_Direction = 7
        End Select
    End If

    Exit Function
    
Error_modMap_Quarter_Direction:
    MsgBox "Error_modMap_Quarter_Direction : " & Err.Description
End Function


Function modMap_Get_Facing_XY(Facing As Integer, ByRef Xpos As Long, ByRef Ypos As Long) As Boolean
    On Error GoTo Error_modMap_Get_Facing_XY
    
    Select Case Facing
        Case 1
            Xpos = Xpos - 1
            Ypos = Ypos + 1
        Case 2
            Ypos = Ypos + 1
        Case 3
            Xpos = Xpos + 1
            Ypos = Ypos + 1
        Case 4
            Xpos = Xpos - 1
        Case 6
            Xpos = Xpos + 1
        Case 7
            Xpos = Xpos - 1
            Ypos = Ypos - 1
        Case 8
            Ypos = Ypos - 1
        Case 9
            Xpos = Xpos + 1
            Ypos = Ypos - 1
    End Select

    If Not Map_Check(Xpos, Ypos) Then Exit Function
    
    modMap_Get_Facing_XY = True
    
    Exit Function
    
Error_modMap_Get_Facing_XY:
    MsgBox "Error_modMap_Get_Facing_XY : " & Err.Description
End Function


Sub modMap_Create_Random()
    On Error GoTo Error_modMap_Create_Random

    modMap_Setup_Init "Random"

    Dim i As Long, j As Long
    Dim x As Long, y As Long
    
'''    ' set to all walls
'''    For i = 0 To 100
'''        For j = 0 To 100
'''            modItem_Add 1, i, j
'''        Next j
'''    Next i

    ' add rooms
    For i = 0 To 90 Step 10
        For j = 0 To 90 Step 10
            
            ' make the room
            For x = 0 To 10
                modItem_Add 1, i, j + x
                modItem_Add 1, i + 10, j + x
                modItem_Add 1, i + x, j
                modItem_Add 1, i + x, j + 10
            Next x
            
        Next j
    Next i
    
    ' add connectors
    For i = 0 To 90 Step 10
        For j = 0 To 90 Step 10
            
            ' add some connectors
            If Rnd > 0.5 Then
                modItem_Delete ItemMap(i, j + 5)
                modItem_Delete ItemMap(i + 1, j + 5)
            End If
            
            If Rnd > 0.5 Then
                modItem_Delete ItemMap(i + 9, j + 5)
                modItem_Delete ItemMap(i + 9, j + 5 + 1)
            End If
            
            If Rnd > 0.5 Then
                modItem_Delete ItemMap(i + 5, j)
                modItem_Delete ItemMap(i + 5 + 1, j)
            End If
            
            If Rnd > 0.5 Then
                modItem_Delete ItemMap(i + 5, j + 9)
                modItem_Delete ItemMap(i + 5, j + 9 + 1)
            End If
            
            ' don't put anything in the first room
            If j <> 0 Or i <> 0 Then
                
                ' add monsters and loot
                Select Case Rnd * 100
                    Case 0 To 25
                        modPlayer_Add 1, i + 5, j + 5
                        modPlayer_Add 1, i + 6, j + 5
                    Case 26 To 50
                        modPlayer_Add 2, i + 5, j + 5
                        modPlayer_Add 2, i + 6, j + 5
                    Case 51 To 70
                        modPlayer_Add 1, i + 5, j + 5
                        modPlayer_Add 1, i + 6, j + 5
                        modPlayer_Add 1, i + 5, j + 6
                        modPlayer_Add 1, i + 6, j + 6
                    Case 71 To 90
                        modPlayer_Add 2, i + 5, j + 5
                        modPlayer_Add 2, i + 6, j + 5
                        modPlayer_Add 2, i + 5, j + 6
                        modPlayer_Add 2, i + 6, j + 6
                    Case 91 To 95
                        modPlayer_Add 1, i + 5, j + 5
                        modPlayer_Add 1, i + 6, j + 5
                        modPlayer_Add 1, i + 5, j + 6
                        modPlayer_Add 1, i + 6, j + 6
                        modPlayer_Add 1, i + 4, j + 5
                        modPlayer_Add 1, i + 5, j + 4
                    Case 96 To 101
                        modPlayer_Add 2, i + 5, j + 5
                        modPlayer_Add 2, i + 6, j + 5
                        modPlayer_Add 2, i + 5, j + 6
                        modPlayer_Add 2, i + 6, j + 6
                        modPlayer_Add 2, i + 5, j + 4
                        modPlayer_Add 2, i + 4, j + 5
                        
                End Select
                
                Select Case Rnd * 100
                    Case 0 To 20
                        modItem_Add 24, i + 5, j + 5
                    Case 21 To 35
                        modItem_Add 25, i + 5, j + 5
                    Case 36
                        modItem_Add 27, i + 5, j + 5
                    Case 37
                        modItem_Add 28, i + 5, j + 5
                    Case 38
                        modItem_Add 29, i + 5, j + 5
                    Case 39
                        modItem_Add 30, i + 5, j + 5
                    Case 40
                        modItem_Add 31, i + 5, j + 5
                    Case 41
                        modItem_Add 32, i + 5, j + 5
                    Case 42
                        modItem_Add 33, i + 5, j + 5
                    Case 43
                        modItem_Add 34, i + 5, j + 5
                        
                End Select
            End If
        Next j
    Next i
    
    modMap_Fix_Walls
    
    Map.Name = "Random"
    Map.StartXpos = 2
    Map.StartYpos = 2

    Exit Sub
    
Error_modMap_Create_Random:
    MsgBox "Error_modMap_Create_Random : " & Err.Description
End Sub
