Attribute VB_Name = "modPlayer"
Option Explicit

Global Const MAX_CARRY As Long = 20


Type PlayerType
    flgInUse As Boolean
    
    ID As Long
    
    Name As String * 100
    
    Xpos As Long
    Ypos As Long
    
    HomeXpos As Long
    HomeYpos As Long
    
    Image As Integer
    Facing As Integer
    
    Level As Long
    LevelXPNeeded As Long
    
    XP As Long
    SpendableXP As Long
    
    Life As Long
    MaxLife As Long
        
    Stamina As Long
    MaxStamina As Long
        
    SightRange As Long
   
    Skill(1 To MAX_SKILLS) As SkillType
   
    LastMove As Single
    LastAttack As Single
   
    flgAttack As Boolean
   
    MobEffect As Long
   
    Gold As Long
   
    Carry(1 To MAX_CARRY) As ItemType
   
    Weapon As Long    ' index into Carry
    Armor As Long     ' index into carry
   
End Type

Global Player() As PlayerType


Sub modPlayer_Init()

    ReDim Player(1 To 20)

End Sub


Sub modPlayer_Setup()
    On Error GoTo Error_modPlayer_Setup
    
    Game.pIndex = modPlayer_Add(0, Map.StartXpos, Map.StartYpos)
    Game.ViewXpos = Map.StartXpos
    Game.ViewYpos = Map.StartYpos
    
    Player(Game.pIndex).Name = InputBox("Enter your players name:")
    
    Player(Game.pIndex).Skill(SKILL_MELEE).flgTrained = True
    Player(Game.pIndex).Skill(SKILL_MELEE).Base = Rnd * 10 + 10
    Player(Game.pIndex).Skill(SKILL_MELEE).XP = 100
       
    Exit Sub
    
Error_modPlayer_Setup:
    MsgBox "Error_modPlayer_Setup : " & Err.Description
End Sub


Function modPlayer_Add(ID As Integer, Xpos As Long, Ypos As Long) As Long
    On Error GoTo Error_modPlayer_Add
    
    ' MAJOR PITFALL!!!!
    ' If you pass as an argument anything that references the Player array
    ' and the function tries to resize the array you will get an array locked error!!!
    ' So use temp variables not tied to player() array to pass
    
    ' make sure in bounds
    If Not Map_Check(Xpos, Ypos) Then Exit Function
        
    ' make sure the spot is free
    If PlayerMap(Xpos, Ypos) <> 0 Then Exit Function
        
    Dim i As Long, pIndex As Long
    
    For i = 1 To UBound(Player)
        If Not Player(i).flgInUse Then
            pIndex = i
            Exit For
        End If
    Next i
    
    If pIndex = 0 Then
        ReDim Preserve Player(1 To UBound(Player) + 10)
        pIndex = UBound(Player) - 9
    End If
    
    Dim EmptyPlayer As PlayerType
    
    Player(pIndex) = EmptyPlayer
    
    Player(pIndex).flgInUse = True
    Player(pIndex).ID = ID
    
    Player(pIndex).Xpos = Xpos
    Player(pIndex).Ypos = Ypos
    Player(pIndex).HomeXpos = Xpos
    Player(pIndex).HomeYpos = Ypos
    
    Player(pIndex).Facing = 2
    
    If ID > 0 Then
        Player(pIndex).Level = MonsterDef(ID).Level
        Player(pIndex).Name = MonsterDef(ID).Name
        Player(pIndex).Image = MonsterDef(ID).Image
        
        Player(pIndex).Life = MonsterDef(ID).Life
        Player(pIndex).MaxLife = MonsterDef(ID).Life
        
        Player(pIndex).Stamina = MonsterDef(ID).Stamina
        Player(pIndex).MaxStamina = MonsterDef(ID).Stamina
        
        Player(pIndex).SightRange = MonsterDef(ID).SightRange
        
        Player(pIndex).Skill(SKILL_MELEE).flgTrained = True
        Player(pIndex).Skill(SKILL_MELEE).Base = MonsterDef(ID).Melee
        
        
    Else
        Player(pIndex).Image = 0
        Player(pIndex).Life = 50
        Player(pIndex).MaxLife = 50
        
        Player(pIndex).Stamina = 100
        Player(pIndex).MaxStamina = 100
        
        Player(pIndex).SightRange = 100
        Player(pIndex).Level = 1
        Player(pIndex).LevelXPNeeded = 1000
    End If
    
    PlayerMap(Xpos, Ypos) = pIndex
    
    modPlayer_Add = pIndex
    
    Exit Function
    
Error_modPlayer_Add:
    MsgBox "Error_modPlayer_Add : " & Err.Description
End Function


Sub modPlayer_Remove(pIndex As Long)
    On Error GoTo Error_modPlayer_Remove

    PlayerMap(Player(pIndex).Xpos, Player(pIndex).Ypos) = 0
    
    Dim EmptyPlayer As PlayerType
    
    Player(pIndex) = EmptyPlayer
    
    If pIndex = Game.pIndex Then
        ' um, game over!!!
        Game.pIndex = 0
    End If

    Exit Sub
    
Error_modPlayer_Remove:
    MsgBox "Error_modPlayer_Remove : " & Err.Description
End Sub


Function modPlayer_Move(pIndex As Long, Direction As Integer, Optional flgUseNewFacing As Boolean = True) As Boolean
    On Error GoTo Error_modPlayer_Move

    If Player(pIndex).LastMove > Timer Then Exit Function

    ' direction is the numeric pad numbers: 8 = up, 4 = left, etc
    
    Dim Xpos As Long, Ypos As Long
    Xpos = Player(pIndex).Xpos
    Ypos = Player(pIndex).Ypos
    
    ' calc the new spot they want to move to
    Select Case Direction
        Case 1
            Xpos = Xpos - 1
            Ypos = Ypos + 1
        Case 2
            'Xpos = Xpos - 1
            Ypos = Ypos + 1
        Case 3
            Xpos = Xpos + 1
            Ypos = Ypos + 1
        Case 4
            Xpos = Xpos - 1
            'Ypos = Ypos + 1
        Case 6
            Xpos = Xpos + 1
            'Ypos = Ypos + 1
        Case 7
            Xpos = Xpos - 1
            Ypos = Ypos - 1
        Case 8
            'Xpos = Xpos - 1
            Ypos = Ypos - 1
        Case 9
            Xpos = Xpos + 1
            Ypos = Ypos - 1
    End Select
   
    ' make sure on the map
    If Not Map_Check(Xpos, Ypos) Then Exit Function
    
    Dim pIndex2 As Long
    ' make sure it is free from players
    pIndex2 = PlayerMap(Xpos, Ypos)
    If pIndex2 > 0 Then
        ' might want to do combat here
        If pIndex = Game.pIndex And flgUseNewFacing Then
            modPlayer_Attack pIndex, pIndex2
        End If
        Exit Function
    End If
    
    ' see if any items there blocking
    Dim iIndex As Long
    iIndex = ItemMap(Xpos, Ypos)
    If iIndex > 0 Then
        If ItemDef(Item(iIndex).ID).flgBlockMovement Then
            
            If flgUseNewFacing Then
                ' do auto action here like open doors
                modPlayer_Auto_Action pIndex, iIndex
            End If
            
            Exit Function
        End If
    End If

    ' do the move
    PlayerMap(Player(pIndex).Xpos, Player(pIndex).Ypos) = 0
    Player(pIndex).Xpos = Xpos
    Player(pIndex).Ypos = Ypos
    PlayerMap(Player(pIndex).Xpos, Player(pIndex).Ypos) = pIndex
    
    If flgUseNewFacing Then
        Player(pIndex).Facing = Direction
    End If
        
    If Game.pIndex = pIndex Then
        modSound_Play SOUND_WALK
    End If
    
    If iIndex > 0 Then modItem_Step_On pIndex, Xpos, Ypos
    
    If pIndex = Game.pIndex Then
        Player(pIndex).LastMove = Timer + 0.1
    End If
    
    modPlayer_Move = True
    
    Exit Function
    
Error_modPlayer_Move:
    MsgBox "Error_modPlayer_Move : " & Err.Description
End Function


Sub modPlayer_Auto_Action(pIndex As Long, iIndex As Long)
    On Error GoTo Error_modPlayer_Auto_Action

    Select Case Item(iIndex).ID
        Case 17
            ' open the door
            Item(iIndex).ID = 18
            modSound_Play SOUND_DOOR_OPEN
            
        Case 18
            ' close the door
            Item(iIndex).ID = 17
            modSound_Play SOUND_DOOR_CLOSE
            
        Case 19
            ' open the door
            Item(iIndex).ID = 20
            modSound_Play SOUND_DOOR_OPEN
            
        Case 20
            ' close the door
            Item(iIndex).ID = 19
            modSound_Play SOUND_DOOR_CLOSE
    
    End Select

    Exit Sub
    
Error_modPlayer_Auto_Action:
    MsgBox "Error_modPlayer_Auto_Action : " & Err.Description
End Sub


Sub modPlayer_Change_Facing(pIndex As Long, num As Integer)
    On Error GoTo Error_modPlayer_Change_Facing


    If num = 1 Then
        '<'
        Select Case Player(pIndex).Facing
            Case 1
                Player(pIndex).Facing = 2
            Case 2
                Player(pIndex).Facing = 3
            Case 3
                Player(pIndex).Facing = 6
            Case 6
                Player(pIndex).Facing = 9
            Case 9
                Player(pIndex).Facing = 8
            Case 8
                Player(pIndex).Facing = 7
            Case 7
                Player(pIndex).Facing = 4
            Case 4
                Player(pIndex).Facing = 1

        End Select
        
    ElseIf num = 2 Then
        '>'
        
        Select Case Player(pIndex).Facing
            Case 1
                Player(pIndex).Facing = 4
            Case 2
                Player(pIndex).Facing = 1
            Case 3
                Player(pIndex).Facing = 2
            Case 6
                Player(pIndex).Facing = 3
            Case 9
                Player(pIndex).Facing = 6
            Case 8
                Player(pIndex).Facing = 9
            Case 7
                Player(pIndex).Facing = 8
            Case 4
                Player(pIndex).Facing = 7

        End Select
    
    End If
   
    Exit Sub
    
Error_modPlayer_Change_Facing:
    MsgBox "Error_modPlayer_Change_Facing : " & Err.Description
End Sub


Sub modPlayer_Attack_Direction(aIndex As Long)
    On Error GoTo Error_modPlayer_Attack_Direction

    Dim Xpos As Long, Ypos As Long
    
    Xpos = Player(aIndex).Xpos
    Ypos = Player(aIndex).Ypos
    
    If Not modMap_Get_Facing_XY(Player(aIndex).Facing, Xpos, Ypos) Then Exit Sub

    Dim dIndex As Long
    
    dIndex = PlayerMap(Xpos, Ypos)
    
    If dIndex > 0 Then
        modPlayer_Attack aIndex, dIndex
    End If

    Exit Sub
    
Error_modPlayer_Attack_Direction:
    MsgBox "Error_modPlayer_Attack_Direction : " & Err.Description
End Sub

Sub modPlayer_Attack(aIndex As Long, dIndex As Long)
    On Error GoTo Error_modPlayer_Attack

    ' aIndex is attacker
    ' dIndex is defender
    
    Dim TT As Single
    TT = Timer
    
    ' see if attacking too fast
    If Player(aIndex).LastAttack > TT Then Exit Sub
    
    
   
    Dim AttackRoll As Long, DefendRoll As Long, Damage As Long, Armor As Long
    
    ' use skills to determine the skill rolls
    AttackRoll = modSkill_Roll(aIndex, SKILL_MELEE)
    DefendRoll = modSkill_Roll(dIndex, SKILL_MELEE)
    
    ' adjust rolls here like burden, etc
        
    AttackRoll = AttackRoll - Player(aIndex).MobEffect
    DefendRoll = DefendRoll - Player(dIndex).MobEffect
    
    ' use stamina and do penalty
    Player(aIndex).Stamina = Player(aIndex).Stamina - 1
    If Player(aIndex).Stamina < 0 Then
        Player(aIndex).Stamina = 0
        
        ' stam penalty
        AttackRoll = AttackRoll * 0.25
    End If
        
    ' use stamina and do penalty
    Player(dIndex).Stamina = Player(dIndex).Stamina - 1
    If Player(dIndex).Stamina < 0 Then
        Player(dIndex).Stamina = 0
        
        ' stam penalty
        DefendRoll = DefendRoll * 0.25
    End If
    
    ' attack sound
    If Player(aIndex).ID > 0 Then
        If MonsterDef(Player(aIndex).ID).SoundID > 0 Then
            modSound_Play MonsterDef(Player(aIndex).ID).SoundID
        End If
    Else
        modSound_Play SOUND_SWISH
    End If
    
    ' see result
    
    If AttackRoll <= DefendRoll Then
        ' defender success
                
        ' add xp
        If Game.pIndex = dIndex Then
            modSkill_Raise_XP dIndex, SKILL_MELEE, Player(aIndex).Level * 10
        End If
                
        ' add messages
        'modMessage_Add Trim$(Player(aIndex).Name) & " attacked " & Trim$(Player(dIndex).Name) & " but missed!", vbYellow
    Else
        ' attacker success
            
        ' calc damage
        Damage = modItem_Weapon_Damage(aIndex)
        
        ' adjust for armor and damage types
        Armor = modItem_Armor(dIndex)
        
        Damage = Damage - Armor
        If Damage < 1 Then Damage = 1
        
        ' add messages
'''        If Game.pIndex = aIndex Then
'''            modMessage_Add Trim$(Player(aIndex).Name) & " attacked " & Trim$(Player(dIndex).Name) & " and hit for " & Damage & " damage!", vbGreen
'''        Else
'''            modMessage_Add Trim$(Player(aIndex).Name) & " attacked " & Trim$(Player(dIndex).Name) & " and hit for " & Damage & " damage!", vbRed
'''        End If
        
        ' make sure damage does not exceed life
        If Damage > Player(dIndex).Life Then
            Damage = Player(dIndex).Life
        End If
                
        ' give xp
        If Game.pIndex = aIndex Then
            modSkill_Raise_XP aIndex, SKILL_MELEE, Player(aIndex).Level * Damage
        End If
                
        ' add the red splat on screen
        modMessage_Add_Damage Player(dIndex).Xpos, Player(dIndex).Ypos, Damage
        
        ' play the hurt sound
        If dIndex = Game.pIndex Then
            modSound_Play SOUND_GRUNT
        End If
        
        ' attack hit sound
        modSound_Play SOUND_PUNCH
        
        ' reduce life
        modPlayer_Reduce_Life dIndex, Damage
   
    End If
    
    Player(aIndex).LastAttack = TT + 1#

    If Player(aIndex).ID > 0 Then
        Player(aIndex).LastAttack = Player(aIndex).LastAttack + Rnd * 0.1
    End If

    Player(aIndex).flgAttack = True
    Player(aIndex).MobEffect = Player(aIndex).MobEffect + 5
    
    If dIndex > 0 Then
        Player(dIndex).flgAttack = True
        Player(dIndex).MobEffect = Player(dIndex).MobEffect + 5
    End If

    Exit Sub
        
Error_modPlayer_Attack:
    MsgBox "Error_modPlayer_Attack : " & Err.Description
End Sub


Function modPlayer_Reduce_Life(pIndex As Long, delta As Long) As Boolean
    On Error GoTo Error_modPlayer_Reduce_Life

    Player(pIndex).Life = Player(pIndex).Life - delta
    
    If Player(pIndex).Life <= 0 Then
        
        ' see if a monster
        If Player(pIndex).ID > 0 Then
            ' drop dead item
            If MonsterDef(Player(pIndex).ID).DeadItemID > 0 Then
                modItem_Add MonsterDef(Player(pIndex).ID).DeadItemID, Player(pIndex).Xpos, Player(pIndex).Ypos
            End If
        End If
        
        ' remove the player
        modPlayer_Remove pIndex
        
        modPlayer_Reduce_Life = True
    End If

    Exit Function
    
Error_modPlayer_Reduce_Life:
    MsgBox "Error_modPlayer_Reduce_Life : " & Err.Description
End Function


Sub modPlayer_Raise_XP(pIndex As Long, XP As Long)
    On Error GoTo Error_modPlayer_Raise_XP

    Player(pIndex).XP = Player(pIndex).XP + XP
    Player(pIndex).SpendableXP = Player(pIndex).SpendableXP + XP / 2
    
    Player(pIndex).LevelXPNeeded = Player(pIndex).LevelXPNeeded - XP
    
    If Player(pIndex).LevelXPNeeded <= 0 Then
        Player(pIndex).Level = Player(pIndex).Level + 1
        
        Player(pIndex).LevelXPNeeded = CLng(Player(pIndex).Level) ^ CLng(4) + ((CLng(Player(pIndex).Level) ^ CLng(3) + CLng(20)) * CLng(50))
        
        If pIndex = Game.pIndex Then
            modMessage_Add Trim$(Player(pIndex).Name) & " has gained level " & Player(pIndex).Level & "!", vbGreen
        End If
        
    End If

    Exit Sub
    
Error_modPlayer_Raise_XP:
    MsgBox "Error_modPlayer_Raise_XP : " & Err.Description
End Sub


Sub modPlayer_Process()
    On Error GoTo Error_modPlayer_Process

    ' this is where you deal with natural healing, poison, food, stam regen, etc
    
    Dim i As Long
    
    For i = 1 To UBound(Player)
        
        If Player(i).flgInUse Then
        
            ' if attacked last round then no regen
            
            If Not Player(i).flgAttack Then
                
                If Player(i).Life < Player(i).MaxLife Then
                    Player(i).Life = Player(i).Life + 1
                End If
            
                If Player(i).Stamina < Player(i).MaxStamina Then
                    Player(i).Stamina = Player(i).Stamina + 4
                    
                    If Player(i).Stamina > Player(i).MaxStamina Then
                        Player(i).Stamina = Player(i).MaxStamina
                    End If
                End If
            
            End If
        
            ' reset it
            Player(i).flgAttack = False
            Player(i).MobEffect = 0
        
        End If
    
    Next i

    Exit Sub
    
Error_modPlayer_Process:
    MsgBox "Error_modPlayer_Process : " & Err.Description
End Sub



